@@ -1,20 +1,7 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 1997 Matthias Kalle Dalheimer (kalle@kde.org)
+/*  This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1997 Matthias Kalle Dalheimer <kalle@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 // Qt5 TODO: re-enable. No point in doing it before, it breaks on QString::fromUtf8(QByteArray), which exists in qt5.
@@ -25,9 +12,11 @@
 
 #include "config-kconfig.h"
 
-#include <QtTest>
+#include <QTest>
 #include <qtemporarydir.h>
 #include <QStandardPaths>
+#include <QSignalSpy>
+#include <QTemporaryFile>
 #include <kdesktopfile.h>
 
 #include <ksharedconfig.h>
@@ -1876,7 +1865,7 @@ void KConfigTest::testThreads()
     // QEXPECT_FAIL triggers race conditions, it should be fixed to use QThreadStorage...
     //futures << QtConcurrent::run(this, &KConfigTest::testDeleteWhenLocalized);
     //futures << QtConcurrent::run(this, &KConfigTest::testEntryMap);
-    for (QFuture<void> f : qAsConst(futures)) { // krazy:exclude=foreach
+    for (QFuture<void> f : qAsConst(futures)) {
         f.waitForFinished();
     }
 }
@@ -1962,3 +1951,30 @@ void KConfigTest::testNotify()
     QCOMPARE(otherWatcherSpy[0][0].value<KConfigGroup>().name(), QStringLiteral("TopLevelGroup"));
     QCOMPARE(otherWatcherSpy[0][1].value<QByteArrayList>(), QByteArrayList({"someGlobalEntry"}));
 }
+
+void KConfigTest::testKdeglobalsVsDefault()
+{
+    // Add testRestore key with global value in kdeglobals
+    KConfig glob(QStringLiteral("kdeglobals"));
+    KConfigGroup generalGlob(&glob, "General");
+    generalGlob.writeEntry("testRestore", "global");
+    QVERIFY(glob.sync());
+
+    KConfig local(QStringLiteral(TEST_SUBDIR "restorerc"));
+    KConfigGroup generalLocal(&local, "General");
+    // Check if we get global and not the default value from cpp (defaultcpp) when reading data from restorerc
+    QCOMPARE(generalLocal.readEntry("testRestore", "defaultcpp"), "global");
+
+    // Add test restore key with restore value in restorerc file
+    generalLocal.writeEntry("testRestore", "restore");
+    QVERIFY(local.sync());
+    local.reparseConfiguration();
+    // We expect to get the value from restorerc file
+    QCOMPARE(generalLocal.readEntry("testRestore", "defaultcpp"), "restore");
+
+    // Revert to default testRestore key and we expect to get default value and not the global one
+    generalLocal.revertToDefault("testRestore");
+    local.sync();
+    local.reparseConfiguration();
+    QCOMPARE(generalLocal.readEntry("testRestore", "defaultcpp"), "defaultcpp");
+}
