@@ -1,23 +1,10 @@
 /*
-   This file is part of the KDE libraries
-   Copyright (c) 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
-   Copyright (c) 1999-2000 Preston Brown <pbrown@kde.org>
-   Copyright (C) 1996-2000 Matthias Kalle Dalheimer <kalle@kde.org>
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
+    SPDX-FileCopyrightText: 1999-2000 Preston Brown <pbrown@kde.org>
+    SPDX-FileCopyrightText: 1996-2000 Matthias Kalle Dalheimer <kalle@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include <kconfigdata.h>
@@ -113,6 +100,10 @@ bool KEntryMap::setEntry(const QByteArray &group, const QByteArray &key, const Q
         k = it.key();
         e = *it;
         //qDebug() << "found existing entry for key" << k;
+        // If overridden entry is global and not default. And it's overridden by a non global
+        if (e.bGlobal && !(options & EntryGlobal) && !k.bDefault) {
+            e.bOverridesGlobal = true;
+        }
     } else {
         // make sure the group marker is in the map
         KEntryMap const *that = this;
@@ -134,6 +125,8 @@ bool KEntryMap::setEntry(const QByteArray &group, const QByteArray &key, const Q
 
     e.mValue = value;
     e.bDirty = e.bDirty || (options & EntryDirty);
+    e.bNotify = e.bNotify || (options & EntryNotify);
+
     e.bGlobal = (options & EntryGlobal); //we can't use || here, because changes to entries in
     //kdeglobals would be written to kdeglobals instead
     //of the local config file, regardless of the globals flag
@@ -269,6 +262,8 @@ bool KEntryMap::getEntryOption(const QMap< KEntryKey, KEntry >::ConstIterator &i
             return it->bDeleted;
         case EntryExpansion:
             return it->bExpand;
+        case EntryNotify:
+            return it->bNotify;
         default:
             break; // fall through
         }
@@ -296,13 +291,16 @@ void KEntryMap::setEntryOption(QMap< KEntryKey, KEntry >::Iterator it, KEntryMap
         case EntryExpansion:
             it->bExpand = bf;
             break;
+        case EntryNotify:
+            it->bNotify = bf;
+            break;
         default:
             break; // fall through
         }
     }
 }
 
-bool KEntryMap::revertEntry(const QByteArray &group, const QByteArray &key, KEntryMap::SearchFlags flags)
+bool KEntryMap::revertEntry(const QByteArray &group, const QByteArray &key, KEntryMap::EntryOptions options, KEntryMap::SearchFlags flags)
 {
     Q_ASSERT((flags & KEntryMap::SearchDefaults) == 0);
     Iterator entry = findEntry(group, key, flags);
@@ -323,6 +321,7 @@ bool KEntryMap::revertEntry(const QByteArray &group, const QByteArray &key, KEnt
         } else {
             entry->mValue = QByteArray();
         }
+        entry->bNotify = entry->bNotify || (options & EntryNotify);
         entry->bDirty = true;
         entry->bReverted = true; // skip it when writing out to disk
 
