@@ -1,24 +1,11 @@
 /*
-   This file is part of the KDE libraries
-   Copyright (c) 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
-   Copyright (c) 1999 Preston Brown <pbrown@kde.org>
-   Copyright (c) 1997 Matthias Kalle Dalheimer <kalle@kde.org>
-   Copyright (c) 2001 Waldo Bastian <bastian@kde.org>
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
+    SPDX-FileCopyrightText: 1999 Preston Brown <pbrown@kde.org>
+    SPDX-FileCopyrightText: 1997 Matthias Kalle Dalheimer <kalle@kde.org>
+    SPDX-FileCopyrightText: 2001 Waldo Bastian <bastian@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KCONFIGGROUP_H
@@ -28,9 +15,9 @@
 
 #include <kconfigcore_export.h>
 
-#include <QtCore/QExplicitlySharedDataPointer>
-#include <QtCore/QVariant>
-#include <QtCore/QStringList>
+#include <QExplicitlySharedDataPointer>
+#include <QVariant>
+#include <QStringList>
 
 class KConfig;
 class KConfigGroupPrivate;
@@ -64,10 +51,14 @@ public:
      * This allows the creation of subgroups by passing another
      * group as @p master.
      *
-     * @p group is the group name encoded in UTF-8.
+     * @param group name of group
      */
     KConfigGroup(KConfigBase *master, const QString &group);
-    /** Overload for KConfigGroup(KConfigBase*,const QString&) */
+    /**
+     * Overload for KConfigGroup(KConfigBase*,const QString&)
+     *
+     * @param group name of group, encoded in UTF-8
+     */
     KConfigGroup(KConfigBase *master, const char *group);
 
     /**
@@ -79,12 +70,20 @@ public:
      * as @p master.
      */
     KConfigGroup(const KConfigBase *master, const QString &group);
-    /** Overload for KConfigGroup(const KConfigBase*,const QString&) */
+    /**
+     * Overload for KConfigGroup(const KConfigBase*,const QString&)
+     *
+     * @param group name of group, encoded in UTF-8
+     */
     KConfigGroup(const KConfigBase *master, const char *group);
 
     /** Overload for KConfigGroup(const KConfigBase*,const QString&) */
     KConfigGroup(const QExplicitlySharedDataPointer<KSharedConfig> &master, const QString &group);
-    /** Overload for KConfigGroup(const KConfigBase*,const QString&) */
+    /**
+     * Overload for KConfigGroup(const KConfigBase*,const QString&)
+     *
+     * @param group name of group, encoded in UTF-8
+     */
     KConfigGroup(const QExplicitlySharedDataPointer<KSharedConfig> &master, const char *group);
 
     /**
@@ -124,13 +123,13 @@ public:
      *
      * Syncs the parent config.
      */
-    bool sync() Q_DECL_OVERRIDE;
+    bool sync() override;
 
     /// @reimp
-    void markAsClean() Q_DECL_OVERRIDE;
+    void markAsClean() override;
 
     /// @reimp
-    AccessMode accessMode() const Q_DECL_OVERRIDE;
+    AccessMode accessMode() const override;
 
     /**
      * Return the config object that this group belongs to
@@ -141,23 +140,28 @@ public:
      */
     const KConfig *config() const;
 
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Changes the group of the object
      *
-     * @deprecated
+     * @deprecated Since 5.0
      * Create another KConfigGroup from the parent of this group instead.
      */
-#ifndef KDE_NO_DEPRECATED
-    KCONFIGCORE_DEPRECATED void changeGroup(const QString &group);
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Create another KConfigGroup from the parent of this group")
+    void changeGroup(const QString &group);
 #endif
+
+#if KCONFIGCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Overload for changeGroup(const QString&)
      *
-     * @deprecated
+     * @param group name of group, encoded in UTF-8
+     *
+     * @deprecated Since 5.0.
      * Create another KConfigGroup from the parent of this group instead.
      */
-#ifndef KDE_NO_DEPRECATED
-    KCONFIGCORE_DEPRECATED void changeGroup(const char *group);
+    KCONFIGCORE_DEPRECATED_VERSION(5, 0, "Create another KConfigGroup from the parent of this group")
+    void changeGroup(const char *group);
 #endif
 
     /**
@@ -203,7 +207,7 @@ public:
     /**
      * @reimp
      */
-    QStringList groupList() const Q_DECL_OVERRIDE;
+    QStringList groupList() const override;
 
     /**
      * Returns a list of keys this group contains
@@ -247,7 +251,10 @@ public:
     {
         return readEntry(key.toUtf8().constData(), aDefault);
     }
-    /** Overload for readEntry(const QString&, const T&) const */
+    /**
+     * Overload for readEntry<T>(const QString&, const T&) const
+     * @param key name of key, encoded in UTF-8
+     */
     template <typename T>
     T readEntry(const char *key, const T &aDefault) const;
 
@@ -261,7 +268,10 @@ public:
      * @see writeEntry(), deleteEntry(), hasKey()
      */
     QVariant readEntry(const QString &key, const QVariant &aDefault) const;
-    /** Overload for readEntry(const QString&, const QVariant&) */
+    /**
+     * Overload for readEntry(const QString&, const QVariant&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QVariant readEntry(const char *key, const QVariant &aDefault) const;
 
     /**
@@ -276,13 +286,19 @@ public:
      * @see readPathEntry(), writeEntry(), deleteEntry(), hasKey()
      */
     QString readEntry(const QString &key, const QString &aDefault) const;
-    /** Overload for readEntry(const QString&, const QString&) */
+    /**
+     * Overload for readEntry(const QString&, const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QString readEntry(const char *key, const QString &aDefault) const;
 
-    /** Overload for readEntry(const QString&, const QString&) */
-    QString readEntry(const QString &key, const char *aDefault = Q_NULLPTR) const;
-    /** Overload for readEntry(const QString&, const QString&) */
-    QString readEntry(const char *key, const char *aDefault = Q_NULLPTR) const;
+    /** Overload for readEntry(const QString&, const QString&) const */
+    QString readEntry(const QString &key, const char *aDefault = nullptr) const;
+    /**
+     * Overload for readEntry(const QString&, const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
+    QString readEntry(const char *key, const char *aDefault = nullptr) const;
 
     /**
      * @copydoc readEntry(const char*, const QStringList&) const
@@ -293,7 +309,10 @@ public:
      *          readEntry(const char*, const QList<T>&) const
      */
     QVariantList readEntry(const QString &key, const QVariantList &aDefault) const;
-    /** Overload for readEntry(const QString&, const QVariantList&) */
+    /**
+     * Overload for readEntry(const QString&, const QVariantList&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QVariantList readEntry(const char *key, const QVariantList &aDefault) const;
 
     /**
@@ -306,7 +325,10 @@ public:
      * @see readXdgListEntry(), writeEntry(), deleteEntry(), hasKey()
      */
     QStringList readEntry(const QString &key, const QStringList &aDefault) const;
-    /** Overload for readEntry(const QString&, const QStringList&) */
+    /**
+     * Overload for readEntry(const QString&, const QStringList&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QStringList readEntry(const char *key, const QStringList &aDefault) const;
 
     /**
@@ -323,7 +345,10 @@ public:
     {
         return readEntry(key.toUtf8().constData(), aDefault);
     }
-    /** Overload for readEntry(const QString&, const QList<T>&) */
+    /**
+     * Overload for readEntry<T>(const QString&, const QList<T>&) const
+     * @param key name of key, encoded in UTF-8
+     */
     template<typename T>
     QList<T> readEntry(const char *key, const QList<T> &aDefault) const;
 
@@ -338,8 +363,11 @@ public:
      * @see readEntry(const QString&, const QStringList&) const
      */
     QStringList readXdgListEntry(const QString &pKey, const QStringList &aDefault = QStringList()) const;
-    /** Overload for readXdgListEntry(const QString&, const QStringList&) */
-    QStringList readXdgListEntry(const char *pKey, const QStringList &aDefault = QStringList()) const;
+    /**
+     * Overload for readXdgListEntry(const QString&, const QStringList&) const
+     * @param key name of key, encoded in UTF-8
+     */
+    QStringList readXdgListEntry(const char *key, const QStringList &aDefault = QStringList()) const;
 
     /**
      * Reads a path
@@ -353,7 +381,10 @@ public:
      * @return The value for this key. Can be QString() if @p aDefault is null.
      */
     QString readPathEntry(const QString &pKey, const QString &aDefault) const;
-    /** Overload for readPathEntry(const QString&, const QString&) */
+    /**
+     * Overload for readPathEntry(const QString&, const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QString readPathEntry(const char *key, const QString &aDefault) const;
 
     /**
@@ -368,7 +399,10 @@ public:
      * @return the list, or @p aDefault if the key does not exist
      */
     QStringList readPathEntry(const QString &pKey, const QStringList &aDefault) const;
-    /** Overload for readPathEntry(const QString&, const QStringList&) */
+    /**
+     * Overload for readPathEntry(const QString&, const QStringList&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QStringList readPathEntry(const char *key, const QStringList &aDefault) const;
 
     /**
@@ -382,7 +416,10 @@ public:
      */
     QString readEntryUntranslated(const QString &pKey,
                                   const QString &aDefault = QString()) const;
-    /** Overload for readEntryUntranslated(const QString&, const QString&) */
+    /**
+     * Overload for readEntryUntranslated(const QString&, const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
     QString readEntryUntranslated(const char *key,
                                   const QString &aDefault = QString()) const;
 
@@ -397,30 +434,45 @@ public:
      */
     void writeEntry(const QString &key, const QVariant &value,
                     WriteConfigFlags pFlags = Normal);
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     void writeEntry(const char *key, const QVariant &value,
                     WriteConfigFlags pFlags = Normal);
 
     /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
     void writeEntry(const QString &key, const QString &value,
                     WriteConfigFlags pFlags = Normal);
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     void writeEntry(const char *key, const QString &value,
                     WriteConfigFlags pFlags = Normal);
 
     /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
     void writeEntry(const QString &key, const QByteArray &value,
                     WriteConfigFlags pFlags = Normal);
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     void writeEntry(const char *key, const QByteArray &value,
                     WriteConfigFlags pFlags = Normal);
 
     /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
     void writeEntry(const QString &key, const char *value, WriteConfigFlags pFlags = Normal);
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     void writeEntry(const char *key, const char *value, WriteConfigFlags pFlags = Normal);
 
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     template <typename T>
     void writeEntry(const char *key, const T &value, WriteConfigFlags pFlags = Normal);
     /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
@@ -433,14 +485,20 @@ public:
     /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
     void writeEntry(const QString &key, const QStringList &value,
                     WriteConfigFlags pFlags = Normal);
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     void writeEntry(const char *key, const QStringList &value,
                     WriteConfigFlags pFlags = Normal);
 
     /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
     void writeEntry(const QString &key, const QVariantList &value,
                     WriteConfigFlags pFlags = Normal);
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     void writeEntry(const char *key, const QVariantList &value,
                     WriteConfigFlags pFlags = Normal);
 
@@ -450,7 +508,10 @@ public:
     {
         writeEntry(key.toUtf8().constData(), value, pFlags);
     }
-    /** Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags) */
+    /**
+     * Overload for writeEntry(const QString&, const QVariant&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
     template <typename T>
     void writeEntry(const char *key, const QList<T> &value, WriteConfigFlags pFlags = Normal);
 
@@ -466,8 +527,11 @@ public:
      */
     void writeXdgListEntry(const QString &pKey, const QStringList &value,
                            WriteConfigFlags pFlags = Normal);
-    /** Overload for writeXdgListEntry(const QString&, const QStringList&, WriteConfigFlags) */
-    void writeXdgListEntry(const char *pKey, const QStringList &value,
+    /**
+     * Overload for writeXdgListEntry(const QString&, const QStringList&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
+    void writeXdgListEntry(const char *key, const QStringList &value,
                            WriteConfigFlags pFlags = Normal);
 
     /**
@@ -485,8 +549,11 @@ public:
      */
     void writePathEntry(const QString &pKey, const QString &path,
                         WriteConfigFlags pFlags = Normal);
-    /** Overload for writePathEntry(const QString&, const QString&, WriteConfigFlags) */
-    void writePathEntry(const char *pKey, const QString &path,
+    /**
+     * Overload for writePathEntry(const QString&, const QString&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
+    void writePathEntry(const char *Key, const QString &path,
                         WriteConfigFlags pFlags = Normal);
 
     /**
@@ -504,8 +571,11 @@ public:
      */
     void writePathEntry(const QString &pKey, const QStringList &value,
                         WriteConfigFlags pFlags = Normal);
-    /** Overload for writePathEntry(const QString&, const QStringList&, WriteConfigFlags) */
-    void writePathEntry(const char *pKey, const QStringList &value,
+    /**
+     * Overload for writePathEntry(const QString&, const QStringList&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
+    void writePathEntry(const char *key, const QStringList &value,
                         WriteConfigFlags pFlags = Normal);
 
     /**
@@ -519,8 +589,11 @@ public:
      * @see deleteGroup(), readEntry(), writeEntry()
      */
     void deleteEntry(const QString &pKey, WriteConfigFlags pFlags = Normal);
-    /** Overload for deleteEntry(const QString&, WriteConfigFlags) */
-    void deleteEntry(const char *pKey, WriteConfigFlags pFlags = Normal);
+    /**
+     * Overload for deleteEntry(const QString&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
+    void deleteEntry(const char *key, WriteConfigFlags pFlags = Normal);
 
     /**
      * Checks whether the key has an entry in this group
@@ -538,7 +611,10 @@ public:
      * @see readEntry()
      */
     bool hasKey(const QString &key) const;
-    /** Overload for hasKey(const QString&) const */
+    /**
+     * Overload for hasKey(const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
     bool hasKey(const char *key) const;
 
     /**
@@ -546,7 +622,7 @@ public:
      *
      * @return @c false if the group may be changed, @c true otherwise
      */
-    bool isImmutable() const Q_DECL_OVERRIDE;
+    bool isImmutable() const override;
 
     /**
      * Checks if it is possible to change the given entry
@@ -559,7 +635,10 @@ public:
      *         group object, @c true otherwise
      */
     bool isEntryImmutable(const QString &key) const;
-    /** Overload for isEntryImmutable(const QString&) const */
+    /**
+     * Overload for isEntryImmutable(const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
     bool isEntryImmutable(const char *key) const;
 
     /**
@@ -578,9 +657,21 @@ public:
      *
      * @param key The key of the entry to revert.
      */
+    // TODO KF6 merge with the other one
     void revertToDefault(const QString &key);
-    /** Overload for revertToDefault(const QString&) */
+    void revertToDefault(const QString &key, WriteConfigFlags pFlag);
+
+    // TODO KF6 merge with the other one
+    /**
+     * Overload for revertToDefault(const QString&)
+     * @param key name of key, encoded in UTF-8
+     */
     void revertToDefault(const char *key);
+    /**
+     * Overload for revertToDefault(const QString&, WriteConfigFlags)
+     * @param key name of key, encoded in UTF-8
+     */
+    void revertToDefault(const char *key, WriteConfigFlags pFlag);
 
     /**
      * Whether a default is specified for an entry in either the
@@ -610,7 +701,10 @@ public:
      *          for @p key in this group, @c false otherwise
      */
     bool hasDefault(const QString &key) const;
-    /** Overload for hasDefault(const QString&) const */
+    /**
+     * Overload for hasDefault(const QString&) const
+     * @param key name of key, encoded in UTF-8
+     */
     bool hasDefault(const char *key) const;
 
     /**
@@ -624,11 +718,11 @@ public:
     QMap<QString, QString> entryMap() const;
 
 protected:
-    bool hasGroupImpl(const QByteArray &group) const Q_DECL_OVERRIDE;
-    KConfigGroup groupImpl(const QByteArray &b) Q_DECL_OVERRIDE;
-    const KConfigGroup groupImpl(const QByteArray &b) const Q_DECL_OVERRIDE;
-    void deleteGroupImpl(const QByteArray &group, WriteConfigFlags flags) Q_DECL_OVERRIDE;
-    bool isGroupImmutableImpl(const QByteArray &aGroup) const Q_DECL_OVERRIDE;
+    bool hasGroupImpl(const QByteArray &group) const override;
+    KConfigGroup groupImpl(const QByteArray &b) override;
+    const KConfigGroup groupImpl(const QByteArray &b) const override;
+    void deleteGroupImpl(const QByteArray &group, WriteConfigFlags flags) override;
+    bool isGroupImmutableImpl(const QByteArray &aGroup) const override;
 
 private:
     QExplicitlySharedDataPointer<KConfigGroupPrivate> d;
@@ -650,10 +744,10 @@ private:
 
 #define KCONFIGGROUP_ENUMERATOR_ERROR(ENUM) \
     "The Qt MetaObject system does not seem to know about \"" ENUM \
-    "\" please use Q_ENUMS or Q_FLAGS to register it."
+    "\" please use Q_ENUM or Q_FLAG to register it."
 
 /**
- * To add support for your own enums in KConfig, you can declare them with Q_ENUMS()
+ * To add support for your own enums in KConfig, you can declare them with Q_ENUM()
  * in a QObject subclass (which will make moc generate the code to turn the
  * enum into a string and vice-versa), and then (in the cpp code)
  * use the macro
@@ -686,7 +780,7 @@ private:
     { group.writeEntry(key, value, flags); }
 
 /**
- * Similar to KCONFIGGROUP_DECLARE_ENUM_QOBJECT but for flags declared with Q_FLAGS()
+ * Similar to KCONFIGGROUP_DECLARE_ENUM_QOBJECT but for flags declared with Q_FLAG()
  * (where multiple values can be set at the same time)
  */
 #define KCONFIGGROUP_DECLARE_FLAGS_QOBJECT(Class, Flags)                            \
@@ -720,7 +814,7 @@ template <typename T>
 T KConfigGroup::readEntry(const char *key, const T &defaultValue) const
 {
     ConversionCheck::to_QVariant<T>();
-    return qvariant_cast<T>(readEntry(key, qVariantFromValue(defaultValue)));
+    return qvariant_cast<T>(readEntry(key, QVariant::fromValue(defaultValue)));
 }
 
 template <typename T>
@@ -731,12 +825,13 @@ QList<T> KConfigGroup::readEntry(const char *key, const QList<T> &defaultValue)
 
     QVariantList data;
 
-    Q_FOREACH (const T &value, defaultValue) {
-        data.append(qVariantFromValue(value));
+    for (const T &value : defaultValue) {
+        data.append(QVariant::fromValue(value));
     }
 
     QList<T> list;
-    Q_FOREACH (const QVariant &value, readEntry<QVariantList>(key, data)) {
+    const auto variantList = readEntry<QVariantList>(key, data);
+    for (const QVariant &value : variantList) {
         Q_ASSERT(value.canConvert<T>());
         list.append(qvariant_cast<T>(value));
     }
@@ -748,7 +843,7 @@ template <typename T>
 void KConfigGroup::writeEntry(const char *key, const T &value, WriteConfigFlags pFlags)
 {
     ConversionCheck::to_QVariant<T>();
-    writeEntry(key, qVariantFromValue(value), pFlags);
+    writeEntry(key, QVariant::fromValue(value), pFlags);
 }
 
 template <typename T>
@@ -757,8 +852,8 @@ void KConfigGroup::writeEntry(const char *key, const QList<T> &list, WriteConfig
     ConversionCheck::to_QVariant<T>();
     ConversionCheck::to_QString<T>();
     QVariantList data;
-    Q_FOREACH (const T &value, list) {
-        data.append(qVariantFromValue(value));
+    for (const T &value : list) {
+        data.append(QVariant::fromValue(value));
     }
 
     writeEntry(key, data, pFlags);
